/*
 * Copyright (C) 2002-2005 Josh A. Beam
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* connect directly with ipv6 */

#ifdef IPV6
#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#ifdef _WIN32
	#include <winsock.h>
#else
	#include <sys/socket.h>
	#include <netinet/in.h>
	#include <netdb.h>
#endif /* _WIN32 */
#include "prtunnel.h"

extern char *proxyhost;
extern unsigned short proxyport;

extern int establish_connection6(char *, unsigned short);

/* connect to hostname:port directly */
static int
direct6_connect_to(struct prt_context *context,
                   char *hostname, unsigned short port,
                   char *username, char *password)
{
	int fd;
	struct hostent *host;

	host = gethostbyname2(hostname, AF_INET6);
	if(!host)
		return -1;

	fd = establish_connection6(host->h_addr, port);
	if(fd == -1)
		return -1;

	return fd;
}

static void
direct6_disconnect(struct prt_context *context)
{
	close(context->localfd);
	close(context->remotefd);
#ifndef _WIN32
	shutdown(context->localfd, SHUT_RDWR);
	shutdown(context->remotefd, SHUT_RDWR);
#endif /* _WIN32 */
}

static int
direct6_local_read(struct prt_context *context, char *buf, int size)
{
	return recv(context->localfd, buf, size, 0);
}

static int
direct6_remote_read(struct prt_context *context, char *buf, int size)
{
	return recv(context->remotefd, buf, size, 0);
}

static int
direct6_local_send(struct prt_context *context, char *buf, int size)
{
	return send(context->localfd, buf, size, 0);
}

static int
direct6_remote_send(struct prt_context *context, char *buf, int size)
{
	return send(context->remotefd, buf, size, 0);
}

void
direct6_set_context(struct prt_context *context)
{
	context->connect = direct6_connect_to;
	context->disconnect = direct6_disconnect;
	context->local_read = direct6_local_read;
	context->local_send = direct6_local_send;
	context->remote_read = direct6_remote_read;
	context->remote_send = direct6_remote_send;

	context->data = NULL;
}
#endif /* IPV6 */
